import { system, world, Entity, Player } from "@minecraft/server";
import { EquipmentSlot } from "@minecraft/server";
export const afterEvents = world.afterEvents;
export const beforeEvents = world.beforeEvents;
export function sleep(ticks = 0) {
    return new Promise((resolve) => system.runTimeout(resolve, ticks));
}
export class Vector {
    x;
    y;
    z;
    static up = { x: 0, y: 1, z: 0 };
    static down = { x: 0, y: -1, z: 0 };
    static forward = { x: 0, y: 0, z: 1 };
    static back = { x: 0, y: 0, z: -1 };
    static left = { x: -1, y: 0, z: 0 };
    static right = { x: 1, y: 0, z: 0 };
    static zero = { x: 0, y: 0, z: 0 };
    constructor(x, y, z) {
        if (typeof x === 'object') {
            this.x = x.x;
            this.y = x.y;
            this.z = x.z;
        }
        else {
            this.x = x;
            this.y = y;
            this.z = z;
        }
    }
    floor() {
        return Vector.floor(this);
    }
    equals(other) {
        return Vector.equals(this, other);
    }
    length() {
        return Vector.length(this);
    }
    multiply(value) {
        // @ts-ignore
        return Vector.multiply(this, value);
    }
    add(other) {
        return Vector.add(this, other);
    }
    subtract(other) {
        return Vector.subtract(this, other);
    }
    cross(other) {
        return Vector.cross(this, other);
    }
    distance(other) {
        return Vector.distance(this, other);
    }
    normalize() {
        return Vector.normalize(this);
    }
    static floor(vector) {
        return new Vector(Math.floor(vector.x), Math.floor(vector.y), Math.floor(vector.z));
    }
    static equals(vectorA, vectorB) {
        return JSON.stringify(Vector.floor(vectorA)) === JSON.stringify(Vector.floor(vectorB));
    }
    static length(vector) {
        return Math.sqrt(vector.x * vector.x + vector.y * vector.y + vector.z * vector.z);
    }
    static multiply(vectorA, value) {
        return new Vector(vectorA.x * (typeof value === 'number' ? value : value.x ?? 1), vectorA.y * (typeof value === 'number' ? value : value.y ?? 1), vectorA.z * (typeof value === 'number' ? value : value.z ?? 1));
    }
    static add(vectorA, vectorB) {
        return new Vector(vectorA.x + (vectorB.x ?? 0), vectorA.y + (vectorB.y ?? 0), vectorA.z + (vectorB.z ?? 0));
    }
    static subtract(vectorA, vectorB) {
        return new Vector(vectorA.x - (vectorB.x ?? 0), vectorA.y - (vectorB.y ?? 0), vectorA.z - (vectorB.z ?? 0));
    }
    static cross(vectorA, vectorB) {
        return new Vector(vectorA.y * vectorB.z - vectorA.z * vectorB.y, vectorA.z * vectorB.x - vectorA.x * vectorB.z, vectorA.x * vectorB.y - vectorA.y * vectorB.x);
    }
    static distance(vectorA, vectorB) {
        return Math.sqrt(Math.pow(vectorA.x - vectorB.x, 2)
            + Math.pow(vectorA.y - vectorB.y, 2)
            + Math.pow(vectorA.z - vectorB.z, 2));
    }
    static normalize(vector) {
        const len = Vector.length(vector);
        return new Vector(vector.x / len, vector.y / len, vector.z / len);
    }
    static rotate(vector, angle) {
        const cos = Math.cos(angle);
        const sin = Math.sin(angle);
        const x = vector.x * cos - vector.z * sin;
        const z = vector.x * sin + vector.z * cos;
        return new Vector(x, vector.y, z);
    }
}
// @ts-ignore
Entity.prototype.isPlayer = function () {
    return this.typeId === "minecraft:player";
};
Object.defineProperties(Entity.prototype, {
    health: {
        get: function () {
            return this.getComponent('health');
        }
    },
    inventory: {
        get: function () {
            return this.getComponent('inventory');
        }
    },
    equippable: {
        get: function () {
            return this.getComponent('equippable');
        }
    }
});
Player.prototype.getMainhand = function () {
    return this.equippable.getEquipment(EquipmentSlot.Mainhand);
};
