import { system } from "@minecraft/server";
import { Vector } from "../../libraries/utils";
import { WeaponManager } from "../../libraries/WeaponManager";
const Axe = new WeaponManager(/lua_ma:(\D*_)?heavy_axe/);
Axe.registerAttack('Visual Slash', (player, target_, _item, damage) => {
    const { dimension, location } = player;
    const head = new Vector(player.getHeadLocation());
    const view = new Vector(player.getViewDirection());
    const distance = Vector.distance(location, target_.location);
    dimension.playSound('sound.lua_ma.sweep_axe', location);
    dimension.spawnParticle(`lua_ma:heavy_slash`, head.add(view.multiply(distance)).add({ y: 0.5 }));
});
const waiting = new Set();
Axe.registerTick('Anti Jump', async (player) => {
    if (waiting.has(player.id) || !player.isSprinting || !player.isJumping || player.isGliding || player.isFalling || player.isInWater || player.isFlying || player.isClimbing)
        return;
    waiting.add(player.id);
    const origin = player.location.y;
    const timer = system.runInterval(() => {
        if (!player.isValid() || player.isOnGround || player.isInWater || player.isGliding || player.isFlying || player.isClimbing || player.isFalling) {
            // console.warn('Cancel');
            waiting.delete(player.id);
            return system.clearRun(timer);
        }
        if (player.location.y - origin < 0.25)
            return;
        player.applyKnockback(0, 0, 0, -0.005);
        waiting.delete(player.id);
        system.clearRun(timer);
    }, 1);
});
